local EffectTemplate = import('/lua/EffectTemplates.lua')
local EfctUtil = import('/lua/EffectUtilities.lua')
local CreateBoneEffects = EfctUtil.CreateBoneEffects
local Util = import('/lua/utilities.lua')
local CollisionDetector = import('/mods/ModularDamage/moddata/CollisionDetector/CollisionDetector_script.lua').CollisionDetector
local oldUnit = Unit

Unit = Class(oldUnit) {

	OnCreate = function(self) 
		oldUnit.OnCreate(self)
		local bp = self:GetBlueprint()
		# initialise a table to store each modules stats
		if bp.DamagableModules then
			if not self.ModuleStats then
				self.ModuleStats = {}
			end
			local bp = self:GetBlueprint()
			for i, data in bp.DamagableModules do
				if not self.ModuleStats[i] then
					self.ModuleStats[i] = {}
					self.ModuleStats[i].CurrentHealth = data.Health
					self.ModuleStats[i].FXTable = {}
					self.ModuleStats[i].FX = {}
					self.ModuleStats[i].Enabled = true
					if not data.NoDamage and self.ModuleStats[i].CurrentHealth ~= 0 and not data.NoManualTarget then
						self.ModuleStats[i].CollisionDetector = CollisionDetector{ parent = self, weapon = i }
					end
				end
				#self:DoTakeModuleDamage(self, 3000,Vector(0.2,0.2,0.2), 'Normal', i)
			end
		end 
-- 		if EntityCategoryContains(categories.COMMAND, self) and self:GetArmy() == 1 then
-- 			SimConExecute('dbg collision')
-- 		end
		
	end,

	OnLayerChange = function(self, new, old)
		local bp = self:GetBlueprint()
		if bp.DamagableModules then
			--LOG('LayerChange')
			for k, v in bp.DamagableModules do
				if not v.NoDamage then
					if self.ModuleStats[k].CollisionDetector then
						self.ModuleStats[k].CollisionDetector:LayerChange()
					end
				end
			end
		end
		oldUnit.OnLayerChange(self,new,old)
	end,
	
	# Apply damage to given module
	DoTakeModuleDamage = function(self, instigator, amount, vector, damageType, imodule)
		if self:IsDead() then return end
		local bp = self:GetBlueprint()
		# Apply damage to module
		if bp.DamagableModules[imodule].NoDamage then return end
		self.ModuleStats[imodule].CurrentHealth = self.ModuleStats[imodule].CurrentHealth - amount
		if self.ModuleStats[imodule].CollisionDetector then
			self.ModuleStats[imodule].CollisionDetector:SetHealth(self,self.ModuleStats[imodule].CurrentHealth)
		end
		if self.ModuleStats[imodule].CurrentHealth <= bp.DamagableModules[imodule].Health * 0.75  and self.ModuleStats[imodule].Enabled then
			self:AddModuleDestroyedEffect(imodule,false,vector)
		end
		# destroy module if it has health less than 0
		#LOG(imodule, ' health ; ',self.ModuleStats[imodule].CurrentHealth)
		if self.ModuleStats[imodule].CurrentHealth <= 0  and self.ModuleStats[imodule].Enabled then
			LOG('Module ', imodule, ' destroyed')
			self.ModuleStats[imodule].CurrentHealth = 0
			self:AddModuleDestroyedEffect(imodule,true,vector,false, self.ModuleStats[imodule].CollisionDetector:GetOrientation())
			self.ModuleStats[imodule].Enabled = false
			self:SetWeaponEnabledByLabel(imodule,false)
			if self.ModuleStats[imodule].CollisionDetector then
				self.ModuleStats[imodule].CollisionDetector:Destroy()
				self.ModuleStats[imodule].CollisionDetector = false
			end
			if bp.DamagableModules[imodule].DestroyWith then
				for k, v in bp.DamagableModules[imodule].DestroyWith do
					--LOG(v)
					self.ModuleStats[v].Enabled = false
					self:SetWeaponEnabledByLabel(v,false)
				end
			end
		end
	end,
	
	SetWeaponEnabledByLabel = function(self, label, enable)
		local wep = self:GetWeaponByLabel(label)
		wep:SetWeaponEnabled(enable)
		wep:AimManipulatorSetEnabled(enable)
	end,
    
	OnCaptured = function(self, captor)
        if self and not self:BeenDestroyed() and captor and not captor:BeenDestroyed() and self:GetAIBrain() ~= captor:GetAIBrain() then
            self:DoOnCapturedCallbacks(captor)
            local newUnitCallbacks = {}
            if self.EventCallbacks.OnCapturedNewUnit then
                newUnitCallbacks = self.EventCallbacks.OnCapturedNewUnit
            end
            local entId = self:GetEntityId()
            local unitEnh = SimUnitEnhancements[entId]
            local captorArmyIndex = captor:GetArmy()
            local captorBrain = false
            
            # For campaigns:
            # We need the brain to ignore army cap when transfering the unit
            # do all necessary steps to set brain to ignore, then un-ignore if necessary the unit cap
            
            if ScenarioInfo.CampaignMode then
                captorBrain = captor:GetAIBrain()
                SetIgnoreArmyUnitCap(captorArmyIndex, true)
            end
            
            local newUnit = ChangeUnitArmy(self, captorArmyIndex)
            
			if self:GetBlueprint().DamagableModules then
				local ModuleStats = table.copy(self.ModuleStats)
				for k, v in newUnit.ModuleStats do
					 newUnit.ModuleStats[k].CollisionDetector:Destroy()
				end
				newUnit.ModuleStats = ModuleStats
				for k, v in newUnit.ModuleStats do
					v.FXTable = {}
					if v.Enabled then
						ModuleStats[k].CollisionDetector = CollisionDetector{ parent = newUnit, weapon = k}
						newUnit.ModuleStats[k].CollisionDetector:SetHealth(newUnit,v.CurrentHealth)
					end
					if v.CurrentHealth < self:GetBlueprint().DamagableModules[k].Health * 0.5 then
						newUnit:AddModuleDestroyedEffect(k,false,false,true)
					end
				end
			end
			
            if ScenarioInfo.CampaignMode and not captorBrain.IgnoreArmyCaps then
                SetIgnoreArmyUnitCap(captorArmyIndex, false)
            end
            
            if unitEnh then
                for k,v in unitEnh do
                    newUnit:CreateEnhancement(v)
                end
            end
            # Because the old unit is lost we cannot call a member function for newUnit callbacks
            for k,cb in newUnitCallbacks do
                if cb then
                    cb(newUnit, captor)
                end
            end
        end
    end,
    
	OnHealthChanged = function(self, new, old)
		# This reports a change in the units health, this only triggers every 25% percent. If it is going up (ie being repaired)
		# then we need to increase each module's health proportionaly as well.
		local bp = self:GetBlueprint()

		if bp.DamagableModules then
			if new > old then
				# Unit is being repaired
				local maxhealth = self:GetMaxHealth()
				for i, data in bp.DamagableModules do
					if data.NoDamage then return end
					self.ModuleStats[i].CurrentHealth = bp.DamagableModules[i].Health * (new + (bp.DamagableModules[i].RepairMult or 0 ))
					if self.ModuleStats[i].CurrentHealth > bp.DamagableModules[i].Health then
						self.ModuleStats[i].CurrentHealth = bp.DamagableModules[i].Health 
					end
					# Check if module needs reenabled
					if not self.ModuleStats[i].Enabled and self.ModuleStats[i].CurrentHealth >= bp.DamagableModules[i].Health *0.5 then
						
						LOG('Renabling Module ', i)
						self.ModuleStats[i].Enabled = true
						if not bp.DamagableModules[i].NoManualTarget then
							self.ModuleStats[i].CollisionDetector = CollisionDetector{ parent = self, weapon = i }
						end
						# remove damage effects
						for k, v in self.ModuleStats[i].FXTable do
							v:Destroy()
						end
						self.ModuleStats[i].FXTable = {}
						local boneTable = bp.DamagableModules[i].FXBones or bp.DamagableModules[i].Bones
						local random_bone = boneTable[Random(1, table.getn(boneTable))]
						local FXScale = bp.DamagableModules[i].FXScale or 1
						
						CreateLightParticleIntel( self, random_bone, self:GetArmy(), 3 * FXScale , 6* FXScale, 'flare_lens_add_02', 'ramp_blue_13' )
						
						for k,bone in bp.DamagableModules[i].Bones do
							self:ShowBone(bone, true)
						end 
						# call enable callbacks
						if bp.DamagableModules[i].DestroyWith then
							for k, v in bp.DamagableModules[i].DestroyWith do
								self.ModuleStats[v].CurrentHealth = 1
								self.ModuleStats[v].Enabled = true
								self:SetWeaponEnabledByLabel(v,true)
							end
						end
						self:SetWeaponEnabledByLabel(i,true)
					elseif self.ModuleStats[i].Enabled and self.ModuleStats[i].CurrentHealth > bp.DamagableModules[i].Health * 0.7 then
						for k, v in self.ModuleStats[i].FXTable do
							v:Destroy()
						end
					end
					# Check max health hasn't been exceeded
					if self.ModuleStats[i].CollisionDetector then
						self.ModuleStats[i].CollisionDetector:SetHealth(self,self.ModuleStats[i].CurrentHealth)
					end
				end
			end
		else
			# no damagable modules defined so treat like a regular unit
			self:ManageDamageEffects(new, old)
		end
	end,

	# Add smoke and fire effects to the bones listed in the blueprint.
	AddModuleDestroyedEffect = function(self, imodule,beenDestroyed,vector,capture,ori)
		if table.getn(self.ModuleStats[imodule].FXTable) == 0 then
			--LOG('Adding destroyed effect')
			local bp = self:GetBlueprint()
	
			FXCount = bp.DamagableModules[imodule].FXCount or 1
			
			FXScale = bp.DamagableModules[imodule].FXScale or 1
			local boneTable = bp.DamagableModules[imodule].FXBones or bp.DamagableModules[imodule].Bones
			
			# apply damage effect to a random selection of bones listed
			local effects1
			local effects2
			local effects3
			local army = self:GetArmy()
			if capture then
				effects1 = self.ModuleStats[imodule].FX['effects1']
				effects2 = self.ModuleStats[imodule].FX['effects2']
				effects3 = self.ModuleStats[imodule].FX['effects3']
			else
				effects1 = self.FxDamage1[Random(1,table.getn(self.FxDamage1))]
				effects2 = self.FxDamage2[Random(1,table.getn(self.FxDamage2))]
				effects3 = self.FxDamage3[Random(1,table.getn(self.FxDamage3))]
				self.ModuleStats[imodule].FX['effects1'] = table.copy(effects1)
				self.ModuleStats[imodule].FX['effects2'] = table.copy(effects2)
				self.ModuleStats[imodule].FX['effects3'] = table.copy(effects3)
			end
			
			if not effects1 or not effects2 or not effects3 then return end
			
			if boneTable then
				for i = 1, FXCount do
					for k, v in effects3 do
						local random_bone = boneTable[Random(1, table.getn(boneTable))]
						local fx = CreateAttachedEmitter(self, random_bone, army, v):ScaleEmitter(FXScale)
						table.insert(self.ModuleStats[imodule].FXTable, fx)
					end
				
					for k, v in effects2 do
						local random_bone = boneTable[Random(1, table.getn(boneTable))]
						local fx = CreateAttachedEmitter(self, random_bone, army, v):ScaleEmitter(FXScale)
						table.insert(self.ModuleStats[imodule].FXTable, fx)
					end
				
					for k, v in effects1 do
						local random_bone = boneTable[Random(1, table.getn(boneTable))]
						local fx = CreateAttachedEmitter(self, random_bone, army, v):ScaleEmitter(FXScale)
						table.insert(self.ModuleStats[imodule].FXTable, fx)
					end
				end
			end
		end
		if beenDestroyed then
			self:ForkThread(self.WeaponExplThread,FXCount,FXScale,imodule,vector,ori)
		end
	end,
	
	# Add explosions to the destroyed weapon on Health & FXScale
	
	WeaponExplThread= function(self,FXCount,FXScale,imodule,vector,ori)
		local bp = self:GetBlueprint()
		
		if bp.DamagableModules[imodule].FXScale >= 3 then
			Debris = EffectTemplate.ExplosionDebrisMed01
			Explosion =EffectTemplate.FireCloudMed01 
			Smoke = EffectTemplate.SmokeMed01
			audio = bp.Audio['WepExplosion2'] or bp.Audio['WepExplosion']
		elseif bp.DamagableModules[imodule].FXScale >= 2 then
			Debris = EffectTemplate.ExplosionDebrisMed01 
			Explosion =EffectTemplate.FireCloudMed01 
			Smoke = EffectTemplate.SmokeSml01
			audio =bp.Audio['WepExplosion'] or bp.Audio['WepExplosion2']
		else
			Explosion =  EffectTemplate.FireCloudSml01
			Debris = EffectTemplate.ExplosionDebrisSml01 
			Smoke = EffectTemplate.SmokeSml01
			audio = bp.Audio['WepExplosion'] or bp.Audio['WepExplosion2']
		end
		local army = self:GetArmy()
		local boneTable = bp.DamagableModules[imodule].FXBones or bp.DamagableModules[imodule].Bones
		if bp.DamagableModules[imodule].HasAnimation then
			self:ForkThread(self.CreateMeshProjectile,boneTable,imodule,vector,ori)
		else
			for k, v in boneTable do
				self:HideBone(v,true)
			end
		end
		self:PlaySound(audio)
		for i = 1, FXCount do
		
			local random_bone = boneTable[Random(1, table.getn(boneTable))]
			CreateBoneEffects( self, random_bone, army, Explosion)
			
			CreateLightParticleIntel( self, random_bone, army, 1* FXScale, 4* FXScale, 'flare_lens_add_02', 'ramp_blue_13' )

			WaitTicks(Random(1,5))

			local random_bone = boneTable[Random(1, table.getn(boneTable))]
			CreateBoneEffects( self, random_bone, army, Debris )
	
			WaitTicks(Random(3,6))
			local random_bone = boneTable[Random(1, table.getn(boneTable))]
			CreateBoneEffects( self, random_bone, army, Smoke )
			
			WaitTicks(Random(2,5))
			
			CreateLightParticleIntel( self, random_bone, army, 1* FXScale, 2* FXScale, 'glow_03', 'ramp_blue_13' )
			
			local random_bone = boneTable[Random(1, table.getn(boneTable))]
			CreateBoneEffects( self, random_bone, army, Explosion)
			
			WaitTicks(Random(3,7))
			local random_bone = boneTable[Random(1, table.getn(boneTable))]
			CreateBoneEffects( self, random_bone, army, Explosion)
		end
		
	end,
	
    CreateMeshProjectile = function(self,boneTable,label,vector,ori)
    	if not self:IsDead() then
	    	local bp = self:GetBlueprint()
	    	local path
	    	if EntityCategoryContains(categories.COMMAND,self) and EntityCategoryContains(categories.UEF,self) and self:HasEnhancement('ResourceAllocation') then
	    		path = '/mods/ModularDamage/moddata/' .. self:GetUnitId() .. '/' .. label  ..'_upgrade_proj.bp'
	    	else
	    		path = '/mods/ModularDamage/moddata/' .. self:GetUnitId() .. '/' .. label  ..'_proj.bp'
	    	end
			local proj = self:CreateProjectileAtBone(path,boneTable[1])

			self.proj = proj
			proj.parent = self
			proj.bone = boneTable[1]
			proj.label = label
			proj:SetScale(bp.Display.UniformScale)
			proj:SetBallisticAcceleration(0, -4.9,0)
			proj:SetVelocity(2*(vector.x),2 + (-vector.y), 2* (vector.z))
			--ori[4] = 0
			--proj:SetOrientation(ori,true)
			--proj.motor = self:FallDown() -- vector.x, -vector.y,vector.z,1)
			--proj.motor:Whack(1,0,0,20,false) -- vector[1], vector[2], vector[3], 1, true)
			WaitTicks(1)
			for k, v in boneTable do
				self:HideBone(v, true)
			end
		end
    end,
    
}
