--*****************************************************************************
--* File: lua/modules/ui/game/unitview.lua
--* Author: Chris Blackwell
--* Summary: Rollover unit view control
--*
--* Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

function UpdateROInfo(bp, constructionBool, buildingUnit, enhancementBool)
    local vetLevel = 0
    focusBool = false
    if bp and not table.find(bp.Categories, 'AIMHELPER') then
    
        local displayIndex = 1
        if enhancementBool then
            controls.roName:SetText(bp.Name)
            controls.roIcon:SetTexture(UIUtil.UIFile('/icons/units/default_icon.dds'))
            controls.roStratIcon:Hide()
            controls.roName:SetColor("FFFF9E06")
            controls.roId:SetColor("FF06ECEC")
            controls.roHealth:SetText("")
            controls.roHealthBar:SetValue(0)
            controls.roHealthBar._bar:SetTexture(UIUtil.UIFile('/game/unit_bmp/bar-02_bmp.dds'))
            displayIndex = SetDisplayLine(displayIndex, controls.roDisplay['mass'], tostring(math.ceil(bp.BuildCostMass)), 'FF00FF00', 2, false)
            displayIndex = SetDisplayLine(displayIndex, controls.roDisplay['energy'], tostring(math.ceil(bp.BuildCostEnergy)), 'FF00FF00', 3, false)
            displayIndex = SetDisplayLine(displayIndex, controls.roDisplay[1], FormatTime(bp.BuildTime), 'FF00FF00', 5, false)
            UpdateVetIcons(0)
            consTrue = true
            ShowROBox(true, focusBool, displayIndex)
        else
            local techLevel = 0
            for i, v in bp.Categories do
                if v == "TECH1" then
                    techLevel = 1
                end
                if v == "TECH2" then
                    techLevel = 2
                end
                if v == "TECH3" then
                    techLevel = 3
                end
            end
            for i = 1, 3 do
                if i <= techLevel then
                    controls.roTechLevelPips[i]:SetTexture(UIUtil.UIFile('/game/icon_tech-level/tech-level_solid.dds'))
                else
                    controls.roTechLevelPips[i]:SetTexture(UIUtil.UIFile('/game/icon_tech-level/tech-level_open.dds'))
                end
            end
            local iconName = GameCommon.GetCachedUnitIconFileNames(bp)
            controls.roIcon:SetTexture(iconName)
            if (bp.StrategicIconName) then
                if DiskGetFileInfo("/textures/ui/common/game/strategicicons/" .. string.lower(bp.StrategicIconName) .. "_over.dds") then
                    controls.roStratIcon:SetTexture("/textures/ui/common/game/strategicicons/" .. string.lower(bp.StrategicIconName) .. "_over.dds")
                    controls.roStratIcon:Show()
                else
                    controls.roStratIcon:SetSolidColor('00000000')
                end
            end
            
            if EntityCategoryContains(categories.CIVILIAN, bp.BlueprintId) then
                if rolloverInfo.customName or bp.General.UnitName then
                    controls.roName:SetText(LOCF("%s", rolloverInfo.customName or bp.General.UnitName))
                end
            else
                if techLevel == 0 then
                    if rolloverInfo.customName or bp.General.UnitName then
                        controls.roName:SetText(LOCF("%s: %s", rolloverInfo.customName or bp.General.UnitName, bp.Description))
                    else
                        controls.roName:SetText(LOCF("%s", bp.Description))
                    end
                else
                    if rolloverInfo.customName or bp.General.UnitName then
                        controls.roName:SetText(LOCF("%s: Tech %d %s", rolloverInfo.customName or bp.General.UnitName, techLevel, bp.Description))
                    else
                        controls.roName:SetText(LOCF("Tech %d %s", techLevel, bp.Description))
                    end
                end
            end

            local function UpdateStatusColor(control, currentval, maxval)
                if control._rangeMax != maxval then
                    control:SetRange(0, maxval)
                end
                if currentval / maxval > .75 then
                    control._bar:SetTexture(UIUtil.UIFile('/game/unit_bmp/bar-02_bmp.dds'))
                elseif currentval / maxval > .25 then
                    control._bar:SetTexture(UIUtil.UIFile('/game/unit_bmp/bar-01_bmp.dds'))
                elseif currentval / maxval > 0 then
                    control._bar:SetTexture(UIUtil.UIFile('/game/unit_bmp/bar-03_bmp.dds'))
                end
                control:SetValue(currentval)
            end
            controls.roName:SetColor("FFFF9E06")
            controls.roId:SetColor("FF06ECEC")
            
            
            if(constructionBool) then
                controls.roHealth:SetText(string.format("%d/%d", bp.Defense.MaxHealth, bp.Defense.MaxHealth))
                controls.roHealth:SetColor("FF02f3e2")
                if controls.roHealthBar._rangeMax != bp.Defense.MaxHealth then
                    controls.roHealthBar:SetRange(0, bp.Defense.MaxHealth)
                end
                controls.roHealthBar:SetValue(bp.Defense.MaxHealth)
            else
                UpdateStatusColor(controls.roHealthBar, rolloverInfo.health, rolloverInfo.maxHealth)
                controls.roHealth:SetText(string.format("%d/%d", rolloverInfo.health, rolloverInfo.maxHealth))
                controls.roHealth:SetColor("FFF0F0F0")
            end
            if(constructionBool) then
            else
                if rolloverInfo.nukeSiloMaxStorageCount then
                    if( rolloverInfo.nukeSiloMaxStorageCount > 0 ) then
                        if( rolloverInfo.nukeSiloBuildCount > 0 ) then
                            displayIndex = SetDisplayLine(displayIndex, controls.roDisplay[displayIndex], tostring(rolloverInfo.nukeSiloStorageCount) .. " / " .. tostring(rolloverInfo.nukeSiloMaxStorageCount) .. " (+" .. tostring(rolloverInfo.nukeSiloBuildCount) .. ")", 'FF9eff19', 1, false)
                        else
                            displayIndex = SetDisplayLine(displayIndex, controls.roDisplay[displayIndex], tostring(rolloverInfo.nukeSiloStorageCount) .. " / " .. tostring(rolloverInfo.nukeSiloMaxStorageCount), 'FF9eff19', 1, false)
                        end
                    end
                end
                if rolloverInfo.tacticalSiloMaxStorageCount then
                    if( rolloverInfo.tacticalSiloMaxStorageCount > 0 ) then
                        if( rolloverInfo.tacticalSiloBuildCount > 0 ) then
                            displayIndex = SetDisplayLine(displayIndex, controls.roDisplay[displayIndex], tostring(rolloverInfo.tacticalSiloStorageCount) .. " / " .. tostring(rolloverInfo.tacticalSiloMaxStorageCount) .. " (+" .. tostring(rolloverInfo.tacticalSiloBuildCount) .. ")", 'FF9eff19', 7, false)
                        else
                            displayIndex = SetDisplayLine(displayIndex, controls.roDisplay[displayIndex], tostring(rolloverInfo.tacticalSiloStorageCount) .. " / " .. tostring(rolloverInfo.tacticalSiloMaxStorageCount), 'FF9eff19', 7, false)
                        end
                    end
                end
            end
            local showAbilities = false
            if controls.AbilityList then
                if bp.Display.Abilities != nil then
                    showAbilities = true
                    for i, v in bp.Display.Abilities do
                        local index = i
                        if controls.AbilityList.Value[index] then
                            controls.AbilityList.Value[index]:SetText(LOC(v))
                        else
                            controls.AbilityList.Value[index] = UIUtil.CreateText( controls.AbilityList, LOC(v), 12, UIUtil.bodyFont)
                            LayoutHelpers.Below(controls.AbilityList.Value[index], controls.AbilityList.Value[index-1])
                        end
                    end
                    local maxWidth = 0
                    for i, v in controls.AbilityList.Value do
                        local index = i
                        if v.Width() > maxWidth then
                            maxWidth = v.Width()
                        end
                        if index > table.getsize(bp.Display.Abilities) then
                            v:SetText("")
                        end
                    end
                    controls.AbilityList.bgmidmiddle.Width:Set(maxWidth)
                    controls.AbilityList.bgmidmiddle.Height:Set(function() return (table.getsize(bp.Display.Abilities) * controls.AbilityList.Value[1].Height()) - 10  end)
                else
                    controls.AbilityList:Hide()
                    for i, v in controls.AbilityList.Value do
                        v:SetText("")
                    end
                end
            end
            if(constructionBool) then
                consTrue = true
                if(buildingUnit) then
                    local timeCost, energyCost, massCost = import('/lua/game.lua').GetConstructEconomyModel(buildingUnit, bp.Economy)
                    if(massCost) then
                        SetDisplayLine(displayIndex, controls.roDisplay['mass'], math.ceil(massCost), "FF00FF00", 2, false)
                    end
                    if(energyCost) then
                        SetDisplayLine(displayIndex, controls.roDisplay['energy'], math.ceil(energyCost), "FF00FF00", 3, false)
                    end
                    if(timeCost) then
                        displayIndex = SetDisplayLine(displayIndex, controls.roDisplay[displayIndex], FormatTime(timeCost), "FF00FF00", 5, false)
                    end
                end
            else
                if(rolloverInfo.massProduced >= 0 and rolloverInfo.massConsumed >= 0) then
                    SetDisplayLine(displayIndex, controls.roDisplay['mass'], NetResources(rolloverInfo.massProduced, rolloverInfo.massConsumed), 'FF00FF00', 2, false)
                else                    
                    SetDisplayLine(displayIndex, controls.roDisplay['mass'], '', 'FF00FF00', 2, false)
                end
                if(rolloverInfo.energyProduced >= 0 and rolloverInfo.energyConsumed >= 0) then
                    SetDisplayLine(displayIndex, controls.roDisplay['energy'], NetResources(rolloverInfo.energyProduced, rolloverInfo.energyConsumed), 'FF00FF00', 3, false)
                else
                    SetDisplayLine(displayIndex, controls.roDisplay['energy'], '', 'FF00FF00', 3, false)
                end
                if(rolloverInfo.kills != 0 and rolloverInfo.kills != nil) then
                    local vetLevel5 = 1000
                    local vetLevel4 = 500
                    local vetLevel3 = 250
                    local vetLevel2 = 100
                    local vetLevel1 = 25
                    if bp.Veteran then
                        vetLevel5 = bp.Veteran['Level5']
                        vetLevel4 = bp.Veteran['Level4']
                        vetLevel3 = bp.Veteran['Level3']
                        vetLevel2 = bp.Veteran['Level2']
                        vetLevel1 = bp.Veteran['Level1']
                    end
                     
                    displayIndex = SetDisplayLine(displayIndex, controls.roDisplay[displayIndex], rolloverInfo.kills, 'FFFF0000', 4, false)
                    if(rolloverInfo.kills >= vetLevel5) then
                        vetLevel = 5
                    elseif(rolloverInfo.kills >= vetLevel4) then
                        vetLevel = 4
                    elseif(rolloverInfo.kills >= vetLevel3) then
                        vetLevel = 3
                    elseif(rolloverInfo.kills >= vetLevel2) then
                        vetLevel = 2
                    elseif(rolloverInfo.kills >= vetLevel1) then
                        vetLevel = 1
                    else
                        vetLevel = 0
                    end
                end
            end
            local focusBP
            if rolloverInfo.focus then
                if __blueprints[rolloverInfo.focus.blueprintId] then
                    focusBP = __blueprints[rolloverInfo.focus.blueprintId]
                    if controls.roFocusBG then
                        if focusBP.General.UnitName != nil then
                            controls.roFocusName:SetText(LOC(focusBP.General.UnitName))
                        else
                            controls.roFocusName:SetText(LOC(focusBP.Description))
                        end
                        UpdateStatusColor(controls.roFocusHealthBar, rolloverInfo.focus.health, rolloverInfo.focus.maxHealth)
                    end
                    local iconFocusName = GameCommon.GetCachedUnitIconFileNames(focusBP)
                    controls.roFocusIcon:SetTexture(iconFocusName)
                    controls.roFocusHealthPercent:SetText(math.ceil(100 * (rolloverInfo.focus.health / rolloverInfo.focus.maxHealth)) .. "%")
                    controls.roFocusHealthPercent:SetColor("FFF0F0F0")
                    focusBool = true
                end
            elseif rolloverInfo.focusUpgrade then
                if __blueprints[rolloverInfo.focusUpgrade.blueprintId] then
                    focusBP = __blueprints[rolloverInfo.focusUpgrade.blueprintId]
                    if controls.roFocusBG then
                        local techString = ''
                        for i, v in focusBP.Categories do
                            if v == "TECH2" then
                                techString = '<LOC _Tech_2>'
                            elseif v == "TECH3" then
                                techString = '<LOC _Tech_3>'
                            end
                        end
                        controls.roFocusName:SetText(LOCF('%s %s', "<LOC _Upgrading_to>", techString))
                        UpdateStatusColor(controls.roFocusHealthBar, rolloverInfo.focusUpgrade.health, rolloverInfo.focusUpgrade.maxHealth)
                    end
                    local iconFocusName = GameCommon.GetCachedUnitIconFileNames(focusBP)
                    controls.roFocusIcon:SetTexture(iconFocusName)
                    controls.roFocusHealthPercent:SetText(math.ceil(100 * (rolloverInfo.focusUpgrade.health / rolloverInfo.focusUpgrade.maxHealth)) .. "%")
                    controls.roFocusHealthPercent:SetColor("FFF0F0F0")
                    focusBool = true
                end
            elseif rolloverInfo.focusProp then
                if __blueprints[rolloverInfo.focusProp.blueprintId] then
                    focusBP = __blueprints[rolloverInfo.focusProp.blueprintId]
                    if controls.roFocusBG then
                        controls.roFocusName:SetText(LOC("<LOC rollover_0000>Reclaiming"))
                        controls.roFocusHealthBar:SetRange(0, 1.0)
                        controls.roFocusHealthBar:SetValue(rolloverInfo.focusProp.fractionComplete)
                        controls.roFocusHealthBar._bar:SetTexture(UIUtil.UIFile('/game/unit_bmp/bar-03_bmp.dds'))
                    end
                    controls.roFocusHealthPercent:SetText(math.ceil(100 * (rolloverInfo.focusProp.fractionComplete / 1.0)) .. "%")
                    controls.roFocusHealthPercent:SetColor("FFF0F0F0")
                    controls.roFocusIcon:SetTexture(UIUtil.UIFile('/game/orders/reclaim_btn_up.dds'))
                    focusBool = true
                end
            else
                focusBool = false
            end
            local showprogBar = false
            if displayIndex <= numRoDisplayFields then
                displayIndex = SetDisplayLine(displayIndex, controls.roDisplay[displayIndex], string.format("%d%%", math.floor((rolloverInfo.health / rolloverInfo.maxHealth) * 100)), 'FF00FF00', 8, true)
            end
            if rolloverInfo.fuelRatio > 0 then
                showprogBar = true
                controls.roProgBar:SetValue(rolloverInfo.fuelRatio)
                if displayIndex <= numRoDisplayFields then
                    displayIndex = SetDisplayLine(displayIndex, controls.roDisplay[displayIndex], string.format("%d%%", math.ceil(rolloverInfo.fuelRatio * 100)), 'ffffd300', 9, true)
                end
            elseif rolloverInfo.workProgress > 0 then
                showprogBar = true
                controls.roProgBar:SetValue(rolloverInfo.workProgress)
                if displayIndex <= numRoDisplayFields then
                    displayIndex = SetDisplayLine(displayIndex, controls.roDisplay[displayIndex], string.format("%d%%", math.ceil(rolloverInfo.workProgress * 100)), 'ffffd300', 10, true)
                end
            end
            ShowROBox(true, focusBool, displayIndex, showprogBar, showAbilities)
            #SetZoomAlpha()
            local faction = ''
            local factionData = import('/lua/factions.lua')
            for i, v in bp.Categories do
                local lcCat = string.lower(v)
                if factionData.FactionIndexMap[lcCat] then
                    faction = lcCat
                    break
                end
            end
            UpdateVetIcons(vetLevel, faction)
            while displayIndex <= numRoDisplayFields do
                displayIndex = SetDisplayLine(displayIndex, controls.roDisplay[displayIndex], '', 'FF000000', 6, true)
            end
        end
    elseif bp and table.find(bp.Categories, 'AIMHELPER') then
           local displayIndex = 1

           controls.roName:SetText(LOCF("%s", 'Weapon'))


            local function UpdateStatusColor(control, currentval, maxval)
                if control._rangeMax != maxval then
                    control:SetRange(0, maxval)
                end
                if currentval / maxval > .75 then
                    control._bar:SetTexture(UIUtil.UIFile('/game/unit_bmp/bar-02_bmp.dds'))
                elseif currentval / maxval > .25 then
                    control._bar:SetTexture(UIUtil.UIFile('/game/unit_bmp/bar-01_bmp.dds'))
                elseif currentval / maxval > 0 then
                    control._bar:SetTexture(UIUtil.UIFile('/game/unit_bmp/bar-03_bmp.dds'))
                end
                control:SetValue(currentval)
            end
            controls.roName:SetColor("FFFF9E06")
            controls.roId:SetColor("FF06ECEC")
            
            
            if(constructionBool) then
                controls.roHealth:SetText(string.format("%d/%d", bp.Defense.MaxHealth, bp.Defense.MaxHealth))
                controls.roHealth:SetColor("FF02f3e2")
                if controls.roHealthBar._rangeMax != bp.Defense.MaxHealth then
                    controls.roHealthBar:SetRange(0, bp.Defense.MaxHealth)
                end
                controls.roHealthBar:SetValue(bp.Defense.MaxHealth)
            else
                UpdateStatusColor(controls.roHealthBar, rolloverInfo.health, rolloverInfo.maxHealth)
                controls.roHealth:SetText(string.format("%d/%d", rolloverInfo.health, rolloverInfo.maxHealth))
                controls.roHealth:SetColor("FFF0F0F0")
            end


            local showAbilities = false

                if(rolloverInfo.massProduced >= 0 and rolloverInfo.massConsumed >= 0) then
                    SetDisplayLine(displayIndex, controls.roDisplay['mass'], NetResources(rolloverInfo.massProduced, rolloverInfo.massConsumed), 'FF00FF00', 2, false)
                else                    
                    SetDisplayLine(displayIndex, controls.roDisplay['mass'], '', 'FF00FF00', 2, false)
                end
                if(rolloverInfo.energyProduced >= 0 and rolloverInfo.energyConsumed >= 0) then
                    SetDisplayLine(displayIndex, controls.roDisplay['energy'], NetResources(rolloverInfo.energyProduced, rolloverInfo.energyConsumed), 'FF00FF00', 3, false)
                else
                    SetDisplayLine(displayIndex, controls.roDisplay['energy'], '', 'FF00FF00', 3, false)
                end

            local focusBP
            if rolloverInfo.focus then
                if __blueprints[rolloverInfo.focus.blueprintId] then
                    focusBP = __blueprints[rolloverInfo.focus.blueprintId]
                    if controls.roFocusBG then
                        if focusBP.General.UnitName != nil then
                            controls.roFocusName:SetText(LOC(focusBP.General.UnitName))
                        else
                            controls.roFocusName:SetText(LOC(focusBP.Description))
                        end
                        UpdateStatusColor(controls.roFocusHealthBar, rolloverInfo.focus.health, rolloverInfo.focus.maxHealth)
                    end
                    local iconFocusName = GameCommon.GetCachedUnitIconFileNames(focusBP)
                    controls.roFocusIcon:SetTexture(iconFocusName)
                    controls.roFocusHealthPercent:SetText(math.ceil(100 * (rolloverInfo.focus.health / rolloverInfo.focus.maxHealth)) .. "%")
                    controls.roFocusHealthPercent:SetColor("FFF0F0F0")
                    focusBool = true
                end
   
            else
                focusBool = false
            end
            local showprogBar = false
            if displayIndex <= numRoDisplayFields then
                displayIndex = SetDisplayLine(displayIndex, controls.roDisplay[displayIndex], string.format("%d%%", math.floor((rolloverInfo.health / rolloverInfo.maxHealth) * 100)), 'FF00FF00', 8, true)
            end

            ShowROBox(true, focusBool, displayIndex, showprogBar, false)
            #SetZoomAlpha()
            local faction = ''
            local factionData = import('/lua/factions.lua')
            for i, v in bp.Categories do
                local lcCat = string.lower(v)
                if factionData.FactionIndexMap[lcCat] then
                    faction = lcCat
                    break
                end
            end
            UpdateVetIcons(0, '')
            while displayIndex <= numRoDisplayFields do
                displayIndex = SetDisplayLine(displayIndex, controls.roDisplay[displayIndex], '', 'FF000000', 6, true)
            end
            controls.roStratIcon:Hide()
			controls.roIcon:Hide()
            for i = 1, 3 do
				controls.roTechLevelPips[i]:Hide()
            end
    else
        if(consTrue) then
            ShowROBox(true, focusBool)
            UpdateVetIcons(vetLevel)
        else
            ShowROBox(false, focusBool)
        end
    end
end
