#****************************************************************************
#**
#**  File     :  /moddata/WeaponCollision/WeaponCollision_script.lua
#**  Author(s): Neruz
#**
#**  Summary  :  WeaponCollision, attaches to weapons and records collisions
#**
#**  Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
#****************************************************************************

local Unit = import('/lua/sim/unit.lua').Unit
local Entity = import('/lua/sim/Entity.lua').Entity

function SetPosThread(AimHelper, unit,label, bone)
	while not AimHelper:IsDead() do
		local pos = unit:GetPosition()
		AimHelper:SetPosition(pos, true)
		local ori = unit:GetOrientation()
		AimHelper:SetOrientation(ori,true)
		WaitTicks(3)
	end
end

CollisionDetector = Class(Entity) {

	OnCreate = function(self,spec)
		self.Trash = TrashBag()
		self.parent = spec.parent
		self.module = spec.weapon
		local bp = self.parent:GetBlueprint().DamagableModules[self.module]
		local bone = bp.FXBones[1] or bp.Bones[1]
		self:AttachTo(self.parent, bone)
		local x,y,z = unpack(bp.Collision['Size'] or {0.2,0.2,0.2})
		local ox,oy,oz = unpack(bp.Collision['Offset'] or {0,0,0})
		self:SetCollisionShape('Box',ox,oy,oz,x,y,z)
		self:SetMaxHealth(bp.Health)
		self:SetHealth(self, bp.Health)
		local pos = self.parent:GetPosition(bone)
		self.AimHelper = CreateUnit2('AimHelper', self.parent:GetArmy(),self.parent:GetCurrentLayer(), pos.x, pos.z, 0)
		self.AimHelper.parent = self
		self.AimHelper.module = self.module
		self.Trash:Add(self.AimHelper)
		self.parent.Trash:Add(self)
		self.AimHelper:SetMaxHealth(bp.Health)
		self.AimHelper:SetHealth(self, bp.Health)
		self.AimHelper.posThread = self.AimHelper:ForkThread(SetPosThread,self,self.module,bone)
	end,
	
	OnDamage = function(self, instigator, amount, vector, damageType)
		self.parent:DoTakeModuleDamage(self,amount, vector, damageType, self.module)
		--self.AimHelper:SetHealth(self, self:GetHealth() )
	end,
	
	OnKilled = function(self, instigator, type, overkillRatio)
		self.Trash:Destroy()
	end,
	
	LayerChange = function(self)
		local bp = self.parent:GetBlueprint().DamagableModules[self.module]
		local bone = bp.FXBones[1] or bp.Bones[1]
		self.AimHelper:Destroy()
		local pos = self.parent:GetPosition(bone)
		self.AimHelper = CreateUnit2('AimHelper', self.parent:GetArmy(),self.parent:GetCurrentLayer(), pos.x, pos.z, 0)
		self.AimHelper.parent = self
		self.AimHelper.module = self.module
		self.Trash:Add(self.AimHelper)
		self.AimHelper:SetMaxHealth(bp.Health)
		self.AimHelper:SetHealth(self, bp.Health)
		self.AimHelper.posThread = self.AimHelper:ForkThread(SetPosThread,self,self.module,bone)
	end,
	
	OnKilledByAimHelper = function(self)
		self.parent:DoTakeDamage(self, self:GetHealth(),Vector(0.2,0.2,0.2),'Normal')
		self.parent:DoTakeModuleDamage(self, self:GetHealth(),Vector(0.2,0.2,0.2), 'Normal', self.module)
	end,
	
	SetHealth = function(self, instigator, health)
		Entity.SetHealth(self, instigator, health)
		if self.AimHelper then
			self.AimHelper:SetHealth(instigator, health)
		end
	end,
	
	Destroy = function(self)
		self.Trash:Destroy()
		Entity.Destroy(self)
	end,
}

TypeClass = CollisionDetector